@@ -1,6 +1,6 @@
 /* discedit.c
  *
- * Copyright (c) 1998-2002  Mike Oliphant <oliphant@gtk.org>
+ * Copyright (c) 1998-2004  Mike Oliphant <grip@nostatic.org>
  *
  *   http://www.nostatic.org/grip
  *
@@ -53,6 +53,8 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
   ID3Genre *id3_genre;
   gint id3_genre_count;
   int len;
+  int dub_size;
+  PangoLayout *layout;
 
   uinfo=&(ginfo->gui_info);
 
@@ -65,9 +67,31 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
   label=gtk_label_new(_("Disc title"));
 
   /* This should be the longest string in the track edit section */
-  len=gdk_string_width(label->style->font,_("Track name"))+5;
+
+
+
+  layout=gtk_widget_create_pango_layout(GTK_WIDGET(label),
+					_("Track name"));
+
+
+  pango_layout_get_size(layout,&len,NULL);
+
+  len/=PANGO_SCALE;
+
+  g_object_unref(layout);
+
+  layout=gtk_widget_create_pango_layout(GTK_WIDGET(label),
+					_("W"));
+
+  pango_layout_get_size(layout,&dub_size,NULL);
+
+  dub_size/=PANGO_SCALE;
+
+  g_object_unref(layout);
+
 
   gtk_widget_set_usize(label,len,0);
+
   gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
   gtk_widget_show(label);
 
@@ -139,7 +163,7 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
   adj=gtk_adjustment_new(0,0,9999,1.0,5.0,0);
 
   uinfo->year_spin_button=gtk_spin_button_new(GTK_ADJUSTMENT(adj),0.5,0);
-  gtk_signal_connect(GTK_OBJECT(uinfo->year_spin_button),"changed",
+  gtk_signal_connect(GTK_OBJECT(uinfo->year_spin_button),"value_changed",
 		     GTK_SIGNAL_FUNC(YearEditChanged),(gpointer)ginfo);
   gtk_box_pack_start(GTK_BOX(hbox),uinfo->year_spin_button,TRUE,TRUE,0);
   gtk_widget_show(uinfo->year_spin_button);
@@ -206,9 +230,12 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
 
   entry=MakeStrEntry(&uinfo->split_chars_entry,ginfo->title_split_chars,
 		     _("Split chars"),5,TRUE);
+
   gtk_widget_set_usize(uinfo->split_chars_entry,
-		       5*gdk_string_width(uinfo->split_chars_entry->
-					  style->font,"W"),0);
+		       5*dub_size,0);
+
+
+
   gtk_box_pack_end(GTK_BOX(hbox),entry,FALSE,FALSE,0);
   gtk_widget_show(entry);
 
@@ -264,12 +291,8 @@ void UpdateMultiArtist(GtkWidget *widget,gpointer data)
   uinfo=&(ginfo->gui_info);
 
   if(!ginfo->ddata.data_multi_artist) {
-    gtk_window_set_policy(GTK_WINDOW(uinfo->app),FALSE,TRUE,
-			  uinfo->minimized||uinfo->keep_min_size);
-
     gtk_widget_hide(uinfo->multi_artist_box);
     UpdateGTK();
-    gtk_window_set_policy(GTK_WINDOW(uinfo->app),FALSE,TRUE,FALSE);
   }
   else {
     gtk_widget_show(uinfo->multi_artist_box);
@@ -285,14 +308,21 @@ void ToggleTrackEdit(GtkWidget *widget,gpointer data)
   uinfo=&(ginfo->gui_info);
 
   if(uinfo->track_edit_visible) {
-    gtk_window_set_policy(GTK_WINDOW(uinfo->app),FALSE,TRUE,
-			  uinfo->minimized||uinfo->keep_min_size);
+    gtk_window_resize(GTK_WINDOW(uinfo->app),
+                      uinfo->win_width,
+                      uinfo->win_height);
+
     gtk_widget_hide(uinfo->track_edit_box);
     UpdateGTK();
-    gtk_window_set_policy(GTK_WINDOW(uinfo->app),FALSE,TRUE,FALSE);
   }
   else {
+    if(uinfo->minimized) MinMax(NULL,(gpointer)ginfo);
+
     gtk_widget_show(uinfo->track_edit_box);
+
+    gtk_window_resize(GTK_WINDOW(uinfo->app),
+                      uinfo->win_width,
+                      uinfo->win_height_edit);
   }
 
   uinfo->track_edit_visible=!uinfo->track_edit_visible;
@@ -300,20 +330,32 @@ void ToggleTrackEdit(GtkWidget *widget,gpointer data)
 
 void SetTitle(GripInfo *ginfo,char *title)
 {
+  g_signal_handlers_block_by_func(G_OBJECT(ginfo->gui_info.title_edit_entry),
+                                  TitleEditChanged,(gpointer)ginfo);
+
   gtk_entry_set_text(GTK_ENTRY(ginfo->gui_info.title_edit_entry),title);
   gtk_entry_set_position(GTK_ENTRY(ginfo->gui_info.title_edit_entry),0);
 
   strcpy(ginfo->ddata.data_title,title);
   gtk_label_set(GTK_LABEL(ginfo->gui_info.disc_name_label),title);
+
+  g_signal_handlers_unblock_by_func(G_OBJECT(ginfo->gui_info.title_edit_entry),
+                                    TitleEditChanged,(gpointer)ginfo);
 }
 
 void SetArtist(GripInfo *ginfo,char *artist)
 {
+  g_signal_handlers_block_by_func(G_OBJECT(ginfo->gui_info.artist_edit_entry),
+                                  ArtistEditChanged,(gpointer)ginfo);
+
   gtk_entry_set_text(GTK_ENTRY(ginfo->gui_info.artist_edit_entry),artist);
   gtk_entry_set_position(GTK_ENTRY(ginfo->gui_info.artist_edit_entry),0);
 
   strcpy(ginfo->ddata.data_artist,artist);
   gtk_label_set(GTK_LABEL(ginfo->gui_info.disc_artist_label),artist);
+
+  g_signal_handlers_unblock_by_func(G_OBJECT(ginfo->gui_info.artist_edit_entry),
+                                    ArtistEditChanged,(gpointer)ginfo);
 }
 
 void SetYear(GripInfo *ginfo,int year)
@@ -340,7 +382,8 @@ static void SaveDiscInfo(GtkWidget *widget,gpointer data)
   ginfo=(GripInfo *)data;
 
   if(ginfo->have_disc) {
-    if(DiscDBWriteDiscData(&(ginfo->disc),&(ginfo->ddata),NULL,TRUE,FALSE)<0)
+    if(DiscDBWriteDiscData(&(ginfo->disc),&(ginfo->ddata),NULL,TRUE,FALSE,
+                           "utf-8")<0)
       DisplayMsg(_("Error saving disc data"));
   }
   else DisplayMsg(_("No disc present"));
@@ -352,10 +395,14 @@ static void TitleEditChanged(GtkWidget *widget,gpointer data)
 
   ginfo=(GripInfo *)data;
 
+  printf("title edit changed [%s]\n",
+         gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.title_edit_entry)));
+
   strcpy(ginfo->ddata.data_title,
-	 gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.title_edit_entry)));
+         gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.title_edit_entry)));
+
   gtk_label_set(GTK_LABEL(ginfo->gui_info.disc_name_label),
-		ginfo->ddata.data_title);
+                ginfo->ddata.data_title);
 }
 
 static void ArtistEditChanged(GtkWidget *widget,gpointer data)
@@ -365,9 +412,10 @@ static void ArtistEditChanged(GtkWidget *widget,gpointer data)
   ginfo=(GripInfo *)data;
 
   strcpy(ginfo->ddata.data_artist,
-	 gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.artist_edit_entry)));
+         gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.artist_edit_entry)));
+
   gtk_label_set(GTK_LABEL(ginfo->gui_info.disc_artist_label),
-		ginfo->ddata.data_artist);
+                ginfo->ddata.data_artist);
 }
 
 static void YearEditChanged(GtkWidget *widget,gpointer data)
@@ -385,26 +433,34 @@ void TrackEditChanged(GtkWidget *widget,gpointer data)
 {
   GripInfo *ginfo;
   char newname[256];
+  GtkTreeIter iter;
+  gint i;
 
   ginfo=(GripInfo *)data;
 
   strcpy(ginfo->ddata.data_track[CURRENT_TRACK].track_name,
-	 gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.track_edit_entry)));
+         gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.track_edit_entry)));
   
   strcpy(ginfo->ddata.data_track[CURRENT_TRACK].track_artist,
-	 gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.
-				      track_artist_edit_entry)));
+         gtk_entry_get_text(GTK_ENTRY(ginfo->gui_info.track_artist_edit_entry)));
   
   if(*ginfo->ddata.data_track[CURRENT_TRACK].track_artist)
     g_snprintf(newname,256,"%02d  %s (%s)",CURRENT_TRACK+1,
-	       ginfo->ddata.data_track[CURRENT_TRACK].track_name,
-	       ginfo->ddata.data_track[CURRENT_TRACK].track_artist);
+               ginfo->ddata.data_track[CURRENT_TRACK].track_name,
+               ginfo->ddata.data_track[CURRENT_TRACK].track_artist);
   else
     g_snprintf(newname,256,"%02d  %s",CURRENT_TRACK+1,
-	       ginfo->ddata.data_track[CURRENT_TRACK].track_name);
-  
-  gtk_clist_set_text(GTK_CLIST(ginfo->gui_info.trackclist),
-		     CURRENT_TRACK,0,newname);
+               ginfo->ddata.data_track[CURRENT_TRACK].track_name);
+
+  gtk_tree_model_get_iter_first(GTK_TREE_MODEL(ginfo->gui_info.track_list_store),
+                                &iter);
+  for(i=0;i<CURRENT_TRACK;i++)
+    gtk_tree_model_iter_next(GTK_TREE_MODEL(ginfo->gui_info.track_list_store),
+                             &iter);
+  gtk_list_store_set(ginfo->gui_info.track_list_store,&iter,
+			 TRACKLIST_TRACK_COL,newname,-1);
+  /*  gtk_clist_set_text(GTK_CLIST(ginfo->gui_info.trackclist),
+      CURRENT_TRACK,0,newname);*/
 }
 
 static void EditNextTrack(GtkWidget *widget,gpointer data)
