@@ -1,6 +1,6 @@
 /* grip.c
  *
- * Copyright (c) 1998-2003  Mike Oliphant <oliphant@gtk.org>
+ * Copyright (c) 1998-2004  Mike Oliphant <grip@nostatic.org>
  *
  *   http://www.nostatic.org/grip
  *
@@ -42,6 +42,7 @@
 #include "parsecfg.h"
 
 static void ReallyDie(gint reply,gpointer data);
+static void MakeStatusPage(GripInfo *ginfo);
 static void DoHelp(GtkWidget *widget,gpointer data);
 static void MakeHelpPage(GripInfo *ginfo);
 static void MakeAboutPage(GripGUI *uinfo);
@@ -51,15 +52,8 @@ static void LoadImages(GripGUI *uinfo);
 static void DoLoadConfig(GripInfo *ginfo);
 void DoSaveConfig(GripInfo *ginfo);
 
-static GnomeHelpMenuEntry main_help_entry={"grip","grip.html"};
-static GnomeHelpMenuEntry cdplay_help_entry={"grip","cdplayer.html"};
-static GnomeHelpMenuEntry rip_help_entry={"grip","ripping.html"};
-static GnomeHelpMenuEntry configure_help_entry={"grip","configure.html"};
-static GnomeHelpMenuEntry faq_help_entry={"grip","faq.html"};
-static GnomeHelpMenuEntry morehelp_help_entry={"grip","morehelp.html"};
-static GnomeHelpMenuEntry bug_help_entry={"grip","bugs.html"};
-
 #define BASE_CFG_ENTRIES \
+{"grip_version",CFG_ENTRY_STRING,256,ginfo->version},\
 {"cd_device",CFG_ENTRY_STRING,256,ginfo->cd_device},\
 {"force_scsi",CFG_ENTRY_STRING,256,ginfo->force_scsi},\
 {"ripexename",CFG_ENTRY_STRING,256,ginfo->ripexename},\
@@ -71,6 +65,7 @@ static GnomeHelpMenuEntry bug_help_entry={"grip","bugs.html"};
 {"dbserver",CFG_ENTRY_STRING,256,ginfo->dbserver.name},\
 {"ripfileformat",CFG_ENTRY_STRING,256,ginfo->ripfileformat},\
 {"mp3fileformat",CFG_ENTRY_STRING,256,ginfo->mp3fileformat},\
+{"mp3extension",CFG_ENTRY_STRING,10,ginfo->mp3extension},\
 {"m3ufileformat",CFG_ENTRY_STRING,256,ginfo->m3ufileformat},\
 {"delete_wavs",CFG_ENTRY_BOOL,0,&ginfo->delete_wavs},\
 {"add_m3u",CFG_ENTRY_BOOL,0,&ginfo->add_m3u},\
@@ -94,6 +89,7 @@ static GnomeHelpMenuEntry bug_help_entry={"grip","bugs.html"};
 {"auto_rip",CFG_ENTRY_BOOL,0,&ginfo->auto_rip},\
 {"eject_after_rip",CFG_ENTRY_BOOL,0,&ginfo->eject_after_rip},\
 {"eject_delay",CFG_ENTRY_INT,0,&ginfo->eject_delay},\
+{"delayed_encoding",CFG_ENTRY_BOOL,0,&ginfo->delayed_encoding},\
 {"delay_before_rip",CFG_ENTRY_BOOL,0,&ginfo->delay_before_rip},\
 {"stop_between_tracks",CFG_ENTRY_BOOL,0,&ginfo->stop_between_tracks},\
 {"beep_after_rip",CFG_ENTRY_BOOL,0,&ginfo->beep_after_rip},\
@@ -103,6 +99,9 @@ static GnomeHelpMenuEntry bug_help_entry={"grip","bugs.html"};
 {"use_proxy_env",CFG_ENTRY_BOOL,0,&ginfo->use_proxy_env},\
 {"db_cgi",CFG_ENTRY_STRING,256,ginfo->dbserver.cgi_prog},\
 {"cddb_submit_email",CFG_ENTRY_STRING,256,ginfo->discdb_submit_email},\
+{"discdb_encoding",CFG_ENTRY_STRING,16,ginfo->discdb_encoding},\
+{"id3_encoding",CFG_ENTRY_STRING,16,ginfo->id3_encoding},\
+{"id3v2_encoding",CFG_ENTRY_STRING,16,ginfo->id3v2_encoding},\
 {"db_use_freedb",CFG_ENTRY_BOOL,0,&ginfo->db_use_freedb},\
 {"dbserver2",CFG_ENTRY_STRING,256,ginfo->dbserver2.name},\
 {"db2_cgi",CFG_ENTRY_STRING,256,ginfo->dbserver2.cgi_prog},\
@@ -116,13 +115,20 @@ static GnomeHelpMenuEntry bug_help_entry={"grip","bugs.html"};
 {"no_underscore",CFG_ENTRY_BOOL,0,&ginfo->sprefs.no_underscore},\
 {"allow_high_bits",CFG_ENTRY_BOOL,0,&ginfo->sprefs.allow_high_bits},\
 {"allow_these_chars",CFG_ENTRY_STRING,256,ginfo->sprefs.allow_these_chars},\
-{"keep_min_size",CFG_ENTRY_BOOL,0,&uinfo->keep_min_size},\
 {"num_cpu",CFG_ENTRY_INT,0,&ginfo->edit_num_cpu},\
 {"kbits_per_sec",CFG_ENTRY_INT,0,&ginfo->kbits_per_sec},\
 {"selected_encoder",CFG_ENTRY_INT,0,&ginfo->selected_encoder},\
 {"selected_ripper",CFG_ENTRY_INT,0,&ginfo->selected_ripper},\
 {"play_mode",CFG_ENTRY_INT,0,&ginfo->play_mode},\
 {"playloop",CFG_ENTRY_BOOL,0,&ginfo->playloop},\
+{"win_width",CFG_ENTRY_INT,0,&uinfo->win_width},\
+{"win_height",CFG_ENTRY_INT,0,&uinfo->win_height},\
+{"win_height_edit",CFG_ENTRY_INT,0,&uinfo->win_height_edit},\
+{"win_width_min",CFG_ENTRY_INT,0,&uinfo->win_width_min},\
+{"win_height_min",CFG_ENTRY_INT,0,&uinfo->win_height_min},\
+{"vol_vis",CFG_ENTRY_BOOL,0,&uinfo->volvis},\
+{"track_edit_vis",CFG_ENTRY_BOOL,0,&uinfo->track_edit_visible},\
+{"track_prog_vis",CFG_ENTRY_BOOL,0,&uinfo->track_prog_visible},\
 {"volume",CFG_ENTRY_INT,0,&ginfo->volume},
 
 #define CDPAR_CFG_ENTRIES \
@@ -146,6 +152,8 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
   GtkWidget *app;
   GripInfo *ginfo;
   GripGUI *uinfo;
+  int major,minor,point;
+  char buf[256];
 
   gnome_window_icon_set_default_from_file(GNOME_ICONDIR"/gripicon.png");
 
@@ -157,7 +165,37 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
 
   uinfo=&(ginfo->gui_info);
   uinfo->app=app;
-  
+  uinfo->status_window=NULL;
+  uinfo->rip_status_window=NULL;
+  uinfo->encode_status_window=NULL;
+  uinfo->track_list=NULL;
+
+  uinfo->win_width=WINWIDTH;
+  uinfo->win_height=WINHEIGHT;
+  uinfo->win_height_edit=WINHEIGHTEDIT;
+  uinfo->win_width_min=MIN_WINWIDTH;
+  uinfo->win_height_min=MIN_WINHEIGHT;
+
+  /*  if(geometry != NULL) {
+    gint x,y,w,h;
+    
+    if(gnome_parse_geometry(geometry, 
+			    &x,&y,&w,&h)) {
+      if(x != -1) {
+	gtk_widget_set_uposition(app,x,y);
+      }
+      
+      if(w != -1) {
+        uinfo->win_width=w;
+        uinfo->win_height=h;
+      }
+    }
+    else {
+      g_error(_("Could not parse geometry string `%s'"), geometry);
+    }
+  }
+  */
+
   if(config_filename && *config_filename)
     g_snprintf(ginfo->config_filename,256,"%s",config_filename);
   else {
@@ -176,33 +214,36 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
   ginfo->do_redirect=!no_redirect;
 
   if(!CDInitDevice(ginfo->cd_device,&(ginfo->disc))) {
-    printf(_("Error: Unable to initialize [%s]\n"),ginfo->cd_device);
-    exit(0);
+    sprintf(buf,_("Error: Unable to initialize [%s]\n"),ginfo->cd_device);
+
+    DisplayMsg(buf);
   }
 
   CDStat(&(ginfo->disc),TRUE);
 
   gtk_window_set_policy(GTK_WINDOW(app),FALSE,TRUE,FALSE);
   gtk_window_set_wmclass(GTK_WINDOW(app),"grip","Grip");
-  gtk_signal_connect(GTK_OBJECT(app),"delete_event",
-                     GTK_SIGNAL_FUNC(GripDie),NULL);
+  g_signal_connect(G_OBJECT(app),"delete_event",
+		   G_CALLBACK(GripDie),NULL);
 
-  /* geometry */
-  if(geometry != NULL) {
-    gint x,y,w,h;
-    
-    if(gnome_parse_geometry(geometry, 
-			    &x,&y,&w,&h)) {
-      if(x != -1) {
-	gtk_widget_set_uposition(app,x,y);
-      }
-      
-      if(w != -1) {
-	gtk_window_set_default_size(GTK_WINDOW(app),w,h);
-      }
+  if(uinfo->minimized) {
+    gtk_widget_set_size_request(GTK_WIDGET(app),MIN_WINWIDTH,
+                                MIN_WINHEIGHT);
+
+    gtk_window_resize(GTK_WINDOW(app),uinfo->win_width_min,
+                                uinfo->win_height_min);
+  }
+  else {
+    gtk_widget_set_size_request(GTK_WIDGET(app),WINWIDTH,
+                                WINHEIGHT);
+
+    if(uinfo->track_edit_visible) {
+      gtk_window_resize(GTK_WINDOW(app),uinfo->win_width,
+                        uinfo->win_height_edit);
     }
     else {
-      g_error(_("Could not parse geometry string `%s'"), geometry);
+      gtk_window_resize(GTK_WINDOW(app),uinfo->win_width,
+                        uinfo->win_height);
     }
   }
 
@@ -219,6 +260,7 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
   MakeTrackPage(ginfo);
   MakeRipPage(ginfo);
   MakeConfigPage(ginfo);
+  MakeStatusPage(ginfo);
   MakeHelpPage(ginfo);
   MakeAboutPage(uinfo);
 
@@ -228,13 +270,18 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
   uinfo->track_edit_box=MakeEditBox(ginfo);
   gtk_box_pack_start(GTK_BOX(uinfo->winbox),uinfo->track_edit_box,
 		     FALSE,FALSE,0);
+  if(uinfo->track_edit_visible) gtk_widget_show(uinfo->track_edit_box);
+
 
   uinfo->playopts=MakePlayOpts(ginfo);
   gtk_box_pack_start(GTK_BOX(uinfo->winbox),uinfo->playopts,FALSE,FALSE,0);
   if(uinfo->track_prog_visible) gtk_widget_show(uinfo->playopts);
  
   uinfo->controls=MakeControls(ginfo);
-  gtk_box_pack_start(GTK_BOX(uinfo->winbox),uinfo->controls,FALSE,FALSE,0);
+  if(uinfo->minimized)
+    gtk_box_pack_start(GTK_BOX(uinfo->winbox),uinfo->controls,TRUE,TRUE,0);
+  else
+    gtk_box_pack_start(GTK_BOX(uinfo->winbox),uinfo->controls,FALSE,FALSE,0);
   gtk_widget_show(uinfo->controls);
   
   gnome_app_set_contents(GNOME_APP(app),uinfo->winbox);
@@ -242,6 +289,20 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
 
   CheckNewDisc(ginfo,FALSE);
 
+  /* Check if we're running this version for the first time */
+  if(strcmp(VERSION,ginfo->version)!=0) {
+    strcpy(ginfo->version,VERSION);
+
+    sscanf(VERSION,"%d.%d.%d",&major,&minor,&point);
+
+    /* Check if we have a dev release */
+    if(minor%2) {
+      DisplayMsg(_("This is a development version of Grip. If you encounter problems, you are encouraged to revert to the latest stable version."));
+    }
+  }
+
+  LogStatus(ginfo,_("Grip started successfully\n"));
+
   return app;
 }
 
@@ -297,13 +358,96 @@ GtkWidget *MakeNewPage(GtkWidget *notebook,char *name)
   return page;
 }
 
+static void MakeStatusPage(GripInfo *ginfo)
+{
+  GtkWidget *status_page;
+  GtkWidget *vbox,*vbox2;
+  GtkWidget *notebook;
+  GtkWidget *page;
+  GtkWidget *label;
+
+  status_page=MakeNewPage(ginfo->gui_info.notebook,_("Status"));
+
+  vbox2=gtk_vbox_new(FALSE,0);
+  notebook=gtk_notebook_new();
+
+  page=gtk_frame_new(NULL);
+
+  vbox=gtk_vbox_new(FALSE,4);
+  gtk_container_border_width(GTK_CONTAINER(vbox),3);
+
+  ginfo->gui_info.status_window=NewStatusWindow(vbox);
+
+  gtk_container_add(GTK_CONTAINER(page),vbox);
+  gtk_widget_show(vbox);
+
+  label=gtk_label_new(_("General"));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),page,label);
+  gtk_widget_show(page);
+
+
+  page=gtk_frame_new(NULL);
+
+  vbox=gtk_vbox_new(FALSE,4);
+  gtk_container_border_width(GTK_CONTAINER(vbox),3);
+
+  ginfo->gui_info.rip_status_window=NewStatusWindow(vbox);
+
+  gtk_container_add(GTK_CONTAINER(page),vbox);
+  gtk_widget_show(vbox);
+
+  label=gtk_label_new(_("Rip"));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),page,label);
+  /*  gtk_widget_show(page);*/
+
+
+  page=gtk_frame_new(NULL);
+
+  vbox=gtk_vbox_new(FALSE,4);
+  gtk_container_border_width(GTK_CONTAINER(vbox),3);
+
+  ginfo->gui_info.encode_status_window=NewStatusWindow(vbox);
+
+  gtk_container_add(GTK_CONTAINER(page),vbox);
+  gtk_widget_show(vbox);
+
+  label=gtk_label_new(_("Encode"));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),page,label);
+  /*  gtk_widget_show(page);*/
+
+
+  gtk_box_pack_start(GTK_BOX(vbox2),notebook,TRUE,TRUE,0);
+  gtk_widget_show(notebook);
+
+  gtk_container_add(GTK_CONTAINER(status_page),vbox2);
+  gtk_widget_show(vbox2);
+}
+
+void LogStatus(GripInfo *ginfo,char *fmt,...)
+{
+  va_list args;
+  char *buf;
+
+  if(!ginfo->gui_info.status_window) return;
+
+  va_start(args,fmt);
+
+  buf=g_strdup_vprintf(fmt,args);
+
+  va_end(args);
+
+  StatusWindowWrite(ginfo->gui_info.status_window,buf);
+
+  g_free(buf);
+}
+
 static void DoHelp(GtkWidget *widget,gpointer data)
 {
-  GnomeHelpMenuEntry *entry;
+  char *section;
 
-  entry=(GnomeHelpMenuEntry *)data;
+  section=(char *)data;
 
-  gnome_help_display(NULL,entry);
+  gnome_help_display("grip.xml",section,NULL);
 }
 
 static void MakeHelpPage(GripInfo *ginfo)
@@ -319,43 +463,43 @@ static void MakeHelpPage(GripInfo *ginfo)
 
   button=gtk_button_new_with_label(_("Table Of Contents"));
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)&main_help_entry);
+		     GTK_SIGNAL_FUNC(DoHelp),NULL);
   gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
   gtk_widget_show(button);
 
   button=gtk_button_new_with_label(_("Playing CDs"));
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)&cdplay_help_entry);
+		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"cdplayer");
   gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
   gtk_widget_show(button);
 
   button=gtk_button_new_with_label(_("Ripping CDs"));
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)&rip_help_entry);
+		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"ripping");
   gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
   gtk_widget_show(button);
 
   button=gtk_button_new_with_label(_("Configuring Grip"));
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)&configure_help_entry);
+		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"configure");
   gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
   gtk_widget_show(button);
 
   button=gtk_button_new_with_label(_("FAQ"));
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)&faq_help_entry);
+		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"faq");
   gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
   gtk_widget_show(button);
 
   button=gtk_button_new_with_label(_("Getting More Help"));
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)&morehelp_help_entry);
+		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"morehelp");
   gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
   gtk_widget_show(button);
 
   button=gtk_button_new_with_label(_("Reporting Bugs"));
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)&bug_help_entry);
+		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"bugs");
   gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
   gtk_widget_show(button);
 
@@ -392,13 +536,13 @@ void MakeAboutPage(GripGUI *uinfo)
 
   vbox2=gtk_vbox_new(TRUE,0);
 
-  sprintf(versionbuf,"Version %s",VERSION);
+  sprintf(versionbuf,_("Version %s"),VERSION);
   label=gtk_label_new(versionbuf);
   gtk_widget_set_style(label,uinfo->style_wb);
   gtk_box_pack_start(GTK_BOX(vbox2),label,FALSE,FALSE,0);
   gtk_widget_show(label);
 
-  label=gtk_label_new("Copyright (c) 1998-2003, Mike Oliphant");
+  label=gtk_label_new("Copyright 1998-2004, Mike Oliphant");
   gtk_widget_set_style(label,uinfo->style_wb);
   gtk_box_pack_start(GTK_BOX(vbox2),label,FALSE,FALSE,0);
   gtk_widget_show(label);
@@ -414,7 +558,7 @@ void MakeAboutPage(GripGUI *uinfo)
 
   button=gtk_button_new_with_label("http://www.nostatic.org/grip");
   gtk_widget_set_style(button,uinfo->style_dark_grey);
-  gtk_widget_set_style(GTK_BUTTON(button)->child,
+  gtk_widget_set_style(GTK_BIN(button)->child,
 		       uinfo->style_dark_grey);
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
 		     GTK_SIGNAL_FUNC(Homepage),NULL);
@@ -535,17 +679,6 @@ void GripUpdate(GtkWidget *app)
   if(!ginfo->poll_interval)
     ginfo->poll_interval=1;
 
-#ifdef GRIPCD
-  if(ginfo->poll_drive && !(secs%ginfo->poll_interval)) {
-    if(!ginfo->have_disc)
-      CheckNewDisc(ginfo,FALSE);
-  }
-
-  if(ginfo->auto_eject_countdown && !(--ginfo->auto_eject_countdown))
-    EjectDisc(&(ginfo->disc));
-
-  UpdateDisplay(ginfo);
-#else
   if(ginfo->ripping|ginfo->encoding) UpdateRipProgress(ginfo);
 
   if(!ginfo->ripping_a_disc) {
@@ -556,7 +689,6 @@ void GripUpdate(GtkWidget *app)
 
     UpdateDisplay(ginfo);
   }
-#endif
 }
 
 void Busy(GripGUI *uinfo)
@@ -590,7 +722,6 @@ static void DoLoadConfig(GripInfo *ginfo)
   outputdir[0]='\0';
 
   uinfo->minimized=FALSE;
-  uinfo->keep_min_size=TRUE;
   uinfo->volvis=FALSE;
   uinfo->track_prog_visible=FALSE;
   uinfo->track_edit_visible=FALSE;
@@ -599,6 +730,8 @@ static void DoLoadConfig(GripInfo *ginfo)
 
   uinfo->id3_genre_item_list=NULL;
 
+  *ginfo->version='\0';
+
   strcpy(ginfo->cd_device,"/dev/cdrom");
   *ginfo->force_scsi='\0';
 
@@ -617,7 +750,7 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->auto_eject_countdown=0;
   ginfo->current_discid=0;
   ginfo->volume=255;
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__)
   ginfo->poll_drive=FALSE;
   ginfo->poll_interval=15;
 #else
@@ -649,6 +782,10 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->db_use_freedb=TRUE;
   *ginfo->user_email='\0';
 
+  strcpy(ginfo->discdb_encoding,"UTF-8");
+  strcpy(ginfo->id3_encoding,"UTF-8");
+  strcpy(ginfo->id3v2_encoding,"UTF-8");
+
   ginfo->local_mode=FALSE;
   ginfo->update_required=FALSE;
   ginfo->looking_up=FALSE;
@@ -666,6 +803,8 @@ static void DoLoadConfig(GripInfo *ginfo)
 
   strcpy(ginfo->title_split_chars,"/");
 
+  ginfo->curr_pipe_fd=-1;
+
   ginfo->num_cpu=1;
   ginfo->ripping=FALSE;
   ginfo->ripping_a_disc=FALSE;
@@ -677,6 +816,8 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->num_wavs=0;
   ginfo->doencode=FALSE;
   ginfo->encode_list=NULL;
+  ginfo->pending_list=NULL;
+  ginfo->delayed_encoding = 0;
   ginfo->do_redirect=TRUE;
   ginfo->selected_ripper=0;
 #ifdef CDPAR
@@ -717,7 +858,8 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->selected_encoder=1;
   strcpy(ginfo->mp3cmdline,"-h -b %b %w %m");
   FindExeInPath("lame", ginfo->mp3exename, sizeof(ginfo->mp3exename));
-  strcpy(ginfo->mp3fileformat,"~/mp3/%A/%d/%n.mp3");
+  strcpy(ginfo->mp3fileformat,"~/mp3/%A/%d/%n.%x");
+  strcpy(ginfo->mp3extension,"mp3");
   ginfo->mp3nice=0;
   *ginfo->mp3_filter_cmd='\0';
   ginfo->delete_wavs=TRUE;
@@ -730,7 +872,7 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->doid3=TRUE;
   ginfo->doid3=FALSE;
   ginfo->tag_mp3_only=TRUE;
-  strcpy(ginfo->id3_comment,"Created by Grip");
+  strcpy(ginfo->id3_comment,_("Created by Grip"));
   *ginfo->cdupdate='\0';
   ginfo->sprefs.no_lower_case=FALSE;
   ginfo->sprefs.allow_high_bits=FALSE;
@@ -757,6 +899,9 @@ static void DoLoadConfig(GripInfo *ginfo)
     DoSaveConfig(ginfo);
   }
 
+  LoadRipperConfig(ginfo,ginfo->selected_ripper);
+  LoadEncoderConfig(ginfo,ginfo->selected_encoder);
+
 #ifndef GRIPCD
   /* Phase out 'outputdir' variable */
 
@@ -835,6 +980,9 @@ void DoSaveConfig(GripInfo *ginfo)
 
   if(!SaveConfig(filename,"GRIP",2,cfg_entries))
     DisplayMsg(_("Error: Unable to save config file"));
+
+  SaveRipperConfig(ginfo,ginfo->selected_ripper);
+  SaveEncoderConfig(ginfo,ginfo->selected_encoder);
 }
 
 /* Shut down stuff (generally before an exec) */
@@ -852,8 +1000,16 @@ void CloseStuff(void *user_data)
   dup2(fd,0);
 
   if(ginfo->do_redirect) {
-    dup2(fd,1);
-    dup2(fd,2);
+    if(ginfo->curr_pipe_fd>0) {
+      dup2(ginfo->curr_pipe_fd,1);
+      dup2(ginfo->curr_pipe_fd,2);
+
+      ginfo->curr_pipe_fd=-1;
+    }
+    else {
+      dup2(fd,1);
+      dup2(fd,2);
+    }
   }
 
   /* Close any other filehandles that might be around */
