@@ -1,6 +1,6 @@
 /* gripcfg.c
  *
- * Copyright (c) 1998-2002  Mike Oliphant <oliphant@gtk.org>
+ * Copyright (c) 1998-2004  Mike Oliphant <grip@nostatic.org>
  *
  *   http://www.nostatic.org/grip
  *
@@ -25,6 +25,7 @@
 #include "grip.h"
 #include "gripcfg.h"
 #include "dialog.h"
+#include "parsecfg.h"
 
 static void UseProxyChanged(GtkWidget *widget,gpointer data);
 static void RipperSelected(GtkWidget *widget,gpointer data);
@@ -46,18 +47,28 @@ static Ripper ripper_defaults[]={
   {"",""}
 };
 
-static MP3Encoder encoder_defaults[]={{"bladeenc","-%b -QUIT %w %m"},
-				      {"lame","-h -b %b %w %m"},
-				      {"l3enc","-br %b %w %m"},
-				      {"xingmp3enc","-B %b -Q %w"},
-				      {"mp3encode","-p 2 -l 3 -b %b %w %m"},
-				      {"gogo","-b %b %w %m"},
-				      {"oggenc","-o %m -a %a -l %d -t %n -b %b %w"},
-				      {"flac","-V -o %m %w"},
-				      {"other",""},
+static MP3Encoder encoder_defaults[]={{"bladeenc","-%b -QUIT %w %m","mp3"},
+				      {"lame","-h -b %b %w %m","mp3"},
+				      {"l3enc","-br %b %w %m","mp3"},
+				      {"xingmp3enc","-B %b -Q %w","mp3"},
+				      {"mp3encode","-p 2 -l 3 -b %b %w %m",
+				       "mp3"},
+				      {"gogo","-b %b %w %m","mp3"},
+				      {"oggenc",
+				       "-o %m -a %a -l %d -t %n -b %b %w -N %t -G %g -d %y",
+				       "ogg"},
+				      {"flac","-V -o %m %w","flac"},
+				      {"other","",""},
 				      {"",""}
 };
 
+static CFGEntry encoder_cfg_entries[]={
+  {"name",CFG_ENTRY_STRING,256,NULL},
+  {"cmdline",CFG_ENTRY_STRING,256,NULL},
+  {"exe",CFG_ENTRY_STRING,256,NULL},
+  {"extension",CFG_ENTRY_STRING,10,NULL}
+};
+
 static void UseProxyChanged(GtkWidget *widget,gpointer data)
 {
   GripInfo *ginfo;
@@ -318,6 +329,11 @@ void MakeConfigPage(GripInfo *ginfo)
   gtk_box_pack_start(GTK_BOX(vbox),check,FALSE,FALSE,0);
   gtk_widget_show(check);
 
+  check=MakeCheckButton (NULL, &ginfo->delayed_encoding,
+                        _("Delay encoding until disc is ripped"));
+  gtk_box_pack_start(GTK_BOX(vbox),check,FALSE,FALSE,0);
+  gtk_widget_show(check);
+
   check=MakeCheckButton(NULL,&ginfo->stop_between_tracks,
 			_("Stop cdrom drive between tracks"));
   gtk_box_pack_start(GTK_BOX(vbox),check,FALSE,FALSE,0);
@@ -403,6 +419,12 @@ void MakeConfigPage(GripInfo *ginfo)
   gtk_box_pack_start(GTK_BOX(vbox),entry,FALSE,FALSE,0);
   gtk_widget_show(entry);
 
+  entry=MakeStrEntry(&(uinfo->mp3extension_entry),ginfo->mp3extension,
+		     _("Encode file extension"),
+		     10,TRUE);
+  gtk_box_pack_start(GTK_BOX(vbox),entry,FALSE,FALSE,0);
+  gtk_widget_show(entry);
+  
   entry=MakeStrEntry(NULL,ginfo->mp3fileformat,_("Encode file format"),
 		     255,TRUE);
   gtk_box_pack_start(GTK_BOX(vbox),entry,FALSE,FALSE,0);
@@ -484,7 +506,7 @@ void MakeConfigPage(GripInfo *ginfo)
   gtk_box_pack_start(GTK_BOX(vbox),check,FALSE,FALSE,0);
   gtk_widget_show(check);
 
-#ifdef HAVE_ID3LIB
+#ifdef HAVE_ID3V2
   check=MakeCheckButton(NULL,&ginfo->doid3v2,
 			_("Add ID3v2 tags to encoded files"));
   gtk_box_pack_start(GTK_BOX(vbox),check,FALSE,FALSE,0);
@@ -500,6 +522,18 @@ void MakeConfigPage(GripInfo *ginfo)
   gtk_box_pack_start(GTK_BOX(vbox),entry,FALSE,FALSE,0);
   gtk_widget_show(entry);
 
+  entry=MakeStrEntry(NULL,ginfo->id3_encoding,
+		     _("ID3v1 Character set encoding"),16,TRUE);
+  gtk_box_pack_start(GTK_BOX(vbox),entry,FALSE,FALSE,0);
+  gtk_widget_show(entry);
+
+#ifdef HAVE_ID3V2
+  entry=MakeStrEntry(NULL,ginfo->id3v2_encoding,
+		     _("ID3v2 Character set encoding"),16,TRUE);
+  gtk_box_pack_start(GTK_BOX(vbox),entry,FALSE,FALSE,0);
+  gtk_widget_show(entry);
+#endif
+
   gtk_container_add(GTK_CONTAINER(page),vbox);
   gtk_widget_show(vbox);
 
@@ -565,6 +599,11 @@ void MakeConfigPage(GripInfo *ginfo)
   gtk_box_pack_start(GTK_BOX(dbvbox),entry,FALSE,FALSE,0);
   gtk_widget_show(entry);
 
+  entry=MakeStrEntry(NULL,ginfo->discdb_encoding,
+		     _("DB Character set encoding"),16,TRUE);
+  gtk_box_pack_start(GTK_BOX(dbvbox),entry,FALSE,FALSE,0);
+  gtk_widget_show(entry);
+
   check=MakeCheckButton(NULL,&ginfo->db_use_freedb,
 			_("Use freedb extensions"));
   gtk_box_pack_start(GTK_BOX(dbvbox),check,FALSE,FALSE,0);
@@ -658,11 +697,6 @@ void MakeConfigPage(GripInfo *ginfo)
   gtk_box_pack_start(GTK_BOX(vbox),entry,FALSE,FALSE,0);
   gtk_widget_show(entry);
 
-  check=MakeCheckButton(NULL,&(ginfo->gui_info.keep_min_size),
-			_("Keep application minimum size"));
-  gtk_box_pack_start(GTK_BOX(vbox),check,FALSE,FALSE,0);
-  gtk_widget_show(check);
-
   gtk_container_add(GTK_CONTAINER(page),vbox);
   gtk_widget_show(vbox);
 
@@ -689,6 +723,8 @@ static void RipperSelected(GtkWidget *widget,gpointer data)
   uinfo=&(ginfo->gui_info);
   rip=(Ripper *)gtk_object_get_user_data(GTK_OBJECT(widget));
 
+  SaveRipperConfig(ginfo,ginfo->selected_ripper);
+
   selected_ripper=rip-ripper_defaults;
 
   /* Don't overwrite if the selection hasn't changed */
@@ -714,16 +750,64 @@ static void RipperSelected(GtkWidget *widget,gpointer data)
 #endif
 
   if(!ginfo->using_builtin_cdp) {
-    if(strcmp(rip->name,"other")) {
-      FindExeInPath(rip->name, buf, sizeof(buf));
-      gtk_entry_set_text(GTK_ENTRY(uinfo->ripexename_entry), buf);
+    if(LoadRipperConfig(ginfo,ginfo->selected_ripper)) {
+      strcpy(buf,ginfo->ripexename);
+      gtk_entry_set_text(GTK_ENTRY(uinfo->ripexename_entry),buf);
+      strcpy(buf,ginfo->ripcmdline);
+      gtk_entry_set_text(GTK_ENTRY(uinfo->ripcmdline_entry),buf);
+    }
+    else {
+      if(strcmp(rip->name,"other")) {
+        FindExeInPath(rip->name, buf, sizeof(buf));
+        gtk_entry_set_text(GTK_ENTRY(uinfo->ripexename_entry), buf);
+      }
+      else gtk_entry_set_text(GTK_ENTRY(uinfo->ripexename_entry),"");
+      
+      gtk_entry_set_text(GTK_ENTRY(uinfo->ripcmdline_entry),rip->cmdline);
     }
-    else gtk_entry_set_text(GTK_ENTRY(uinfo->ripexename_entry),"");
-    
-    gtk_entry_set_text(GTK_ENTRY(uinfo->ripcmdline_entry),rip->cmdline);
   }
 }
 
+#define RIP_CFG_ENTRIES \
+    {"exe",CFG_ENTRY_STRING,256,ginfo->ripexename},\
+    {"cmdline",CFG_ENTRY_STRING,256,ginfo->ripcmdline},\
+    {"",CFG_ENTRY_LAST,0,NULL}
+
+gboolean LoadRipperConfig(GripInfo *ginfo,int ripcfg)
+{
+  char buf[256];
+  CFGEntry rip_cfg_entries[]={
+    RIP_CFG_ENTRIES
+  };
+
+#ifdef CDPAR
+  if(ripcfg==0) return;
+#endif
+
+  sprintf(buf,"%s/%s-%s",getenv("HOME"),ginfo->config_filename,
+          ripper_defaults[ripcfg].name);
+
+  return (LoadConfig(buf,"GRIP",2,2,rip_cfg_entries)==1);
+}
+
+void SaveRipperConfig(GripInfo *ginfo,int ripcfg)
+{
+  char buf[256];
+  CFGEntry rip_cfg_entries[]={
+    RIP_CFG_ENTRIES
+  };
+
+#ifdef CDPAR
+  if(ripcfg==0) return;
+#endif
+
+  sprintf(buf,"%s/%s-%s",getenv("HOME"),ginfo->config_filename,
+          ripper_defaults[ripcfg].name);
+
+  if(!SaveConfig(buf,"GRIP",2,rip_cfg_entries))
+    DisplayMsg(_("Error: Unable to save ripper config"));
+}
+
 static void EncoderSelected(GtkWidget *widget,gpointer data)
 {
   GripInfo *ginfo;
@@ -735,15 +819,63 @@ static void EncoderSelected(GtkWidget *widget,gpointer data)
   uinfo=&(ginfo->gui_info);
   enc=(MP3Encoder *)gtk_object_get_user_data(GTK_OBJECT(widget));
 
-  if(strcmp(enc->name,"other")) {
-    FindExeInPath(enc->name, buf, sizeof(buf));
-    gtk_entry_set_text(GTK_ENTRY(uinfo->mp3exename_entry),buf);
-  }
-  else gtk_entry_set_text(GTK_ENTRY(uinfo->mp3exename_entry),"");
-
-  gtk_entry_set_text(GTK_ENTRY(uinfo->mp3cmdline_entry),enc->cmdline);
+  SaveEncoderConfig(ginfo,ginfo->selected_encoder);
 
   ginfo->selected_encoder=enc-encoder_defaults;
+
+  if(LoadEncoderConfig(ginfo,ginfo->selected_encoder)) {
+    strcpy(buf,ginfo->mp3exename);
+    gtk_entry_set_text(GTK_ENTRY(uinfo->mp3exename_entry),buf);
+
+    strcpy(buf,ginfo->mp3cmdline);
+    gtk_entry_set_text(GTK_ENTRY(uinfo->mp3cmdline_entry),buf);
+
+    strcpy(buf,ginfo->mp3extension);
+    gtk_entry_set_text(GTK_ENTRY(uinfo->mp3extension_entry),buf);    
+  }
+  else {
+    if(strcmp(enc->name,"other")) {
+      FindExeInPath(enc->name, buf, sizeof(buf));
+      gtk_entry_set_text(GTK_ENTRY(uinfo->mp3exename_entry),buf);
+    }
+    else gtk_entry_set_text(GTK_ENTRY(uinfo->mp3exename_entry),"");
+    
+    gtk_entry_set_text(GTK_ENTRY(uinfo->mp3cmdline_entry),enc->cmdline);
+    gtk_entry_set_text(GTK_ENTRY(uinfo->mp3extension_entry),enc->extension);
+  }
+}
+
+#define ENCODE_CFG_ENTRIES \
+    {"exe",CFG_ENTRY_STRING,256,ginfo->mp3exename},\
+    {"cmdline",CFG_ENTRY_STRING,256,ginfo->mp3cmdline},\
+    {"extension",CFG_ENTRY_STRING,10,ginfo->mp3extension},\
+    {"",CFG_ENTRY_LAST,0,NULL}
+
+gboolean LoadEncoderConfig(GripInfo *ginfo,int encodecfg)
+{
+  char buf[256];
+  CFGEntry encode_cfg_entries[]={
+    ENCODE_CFG_ENTRIES
+  };
+
+  sprintf(buf,"%s/%s-%s",getenv("HOME"),ginfo->config_filename,
+          encoder_defaults[encodecfg].name);
+
+  return (LoadConfig(buf,"GRIP",2,2,encode_cfg_entries)==1);
+}
+
+void SaveEncoderConfig(GripInfo *ginfo,int encodecfg)
+{
+  char buf[256];
+  CFGEntry encode_cfg_entries[]={
+    ENCODE_CFG_ENTRIES
+  };
+
+  sprintf(buf,"%s/%s-%s",getenv("HOME"),ginfo->config_filename,
+          encoder_defaults[encodecfg].name);
+
+  if(!SaveConfig(buf,"GRIP",2,encode_cfg_entries))
+    DisplayMsg(_("Error: Unable to save encoder config"));
 }
 
 void FindExeInPath(char *exename, char *buf, int bsize)
@@ -751,14 +883,18 @@ void FindExeInPath(char *exename, char *buf, int bsize)
   char *path;
   static char **PATH = 0;
 
-  if (!PATH) {
-    char *env = g_getenv("PATH");
+  if(!PATH) {
+    const char *env = g_getenv("PATH");
+
     PATH = g_strsplit(env ? env : "/usr/local/bin:/usr/bin:/bin", ":", 0);
   }
+
   path = FindExe(exename, PATH);
-  if (!path) {
+
+  if(!path) {
     g_snprintf(buf, bsize, "%s", exename);
-  } else {
+  }
+  else {
     g_snprintf(buf, bsize, "%s/%s", path, exename);
   }
 }
