@@ -1,6 +1,6 @@
 /* rip.c
  *
- * Copyright (c) 1998-2002  Mike Oliphant <oliphant@gtk.org>
+ * Copyright (c) 1998-2004  Mike Oliphant <grip@nostatic.org>
  *
  *   http://www.nostatic.org/grip
  *
@@ -27,7 +27,7 @@
 #elif defined (HAVE_SYS_VFS_H)
 #include <sys/vfs.h>
 #endif
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__)
 #include <sys/param.h>
 #include <sys/mount.h>
 #endif
@@ -48,10 +48,7 @@
 #include "common.h"
 #ifdef CDPAR
 #include "gain_analysis.h"
-#define size32 gint32
-#define size16 gint16
-#include <cdda_interface.h>
-#include <cdda_paranoia.h>
+#include "cdpar.h"
 extern int rip_smile_level;
 #endif
 
@@ -91,6 +88,7 @@ void MakeRipPage(GripInfo *ginfo)
   GtkWidget *partial_rip_frame;
   int mycpu;
   int label_width;
+  PangoLayout *layout;
 
   uinfo=&(ginfo->gui_info);
 
@@ -206,8 +204,17 @@ void MakeRipPage(GripInfo *ginfo)
   uinfo->rip_prog_label=gtk_label_new(_("Rip: Idle"));
 
   /* This should be the largest this string can get */
-  label_width=gdk_string_width(uinfo->app->style->font,
-			       _("Enc: Trk 99 (99.9x)"))+20;
+
+  layout=gtk_widget_create_pango_layout(GTK_WIDGET(uinfo->app),
+					_("Enc: Trk 99 (99.9x)"));
+
+
+  pango_layout_get_size(layout,&label_width,NULL);
+
+  label_width/=PANGO_SCALE;
+
+  g_object_unref(layout);
+
 
   gtk_widget_set_usize(uinfo->rip_prog_label,label_width,0);
   gtk_box_pack_start(GTK_BOX(hbox),uinfo->rip_prog_label,FALSE,FALSE,0);
@@ -370,7 +377,8 @@ static void AddSQLEntry(GripInfo *ginfo,EncodeTrack *enc_track)
   g_snprintf(playtime_str,6,"%d:%d",enc_track->mins,enc_track->secs);
   g_snprintf(year_str,5,"%d",enc_track->song_year);
 
-  Debug("Inserting track %d into the ddj database\n",enc_track->track_num);
+  LogStatus(ginfo,_("Inserting track %d into the ddj database\n"),
+	    enc_track->track_num);
 
   sqlpid=fork();
 
@@ -566,6 +574,8 @@ static gboolean AddM3U(GripInfo *ginfo)
   char m3unam[PATH_MAX];
   char *relnam;
   GString *str;
+  char *conv_str;
+  gsize rb,wb;
 
   if(!ginfo->have_disc) return FALSE;
   
@@ -577,15 +587,21 @@ static gboolean AddM3U(GripInfo *ginfo)
   TranslateString(ginfo->m3ufileformat,str,TranslateSwitch,
 		    &enc_track,TRUE,&(ginfo->sprefs));
 
-  g_snprintf(m3unam,PATH_MAX,"%s",str->str);
+  conv_str=g_filename_from_utf8(str->str,strlen(str->str),&rb,&wb,NULL);
 
-  MakeDirs(str->str);
+  if(!conv_str)
+    conv_str=g_strdup(str->str);
 
-  fp=fopen(str->str, "w");
+  g_snprintf(m3unam,PATH_MAX,"%s",conv_str);
+
+  MakeDirs(conv_str);
+
+  fp=fopen(conv_str, "w");
   if(fp==NULL) {
     DisplayMsg(_("Error: can't open m3u file\n"));
     return FALSE;
   }
+  g_free(conv_str);
   
   for(i=0;i<ginfo->disc.num_tracks;i++) {
     /* Only add to the m3u if the track is selected for ripping */
@@ -596,13 +612,19 @@ static gboolean AddM3U(GripInfo *ginfo)
       TranslateString(ginfo->mp3fileformat,str,TranslateSwitch,
 		      &enc_track,TRUE,&(ginfo->sprefs));
 
+      conv_str=g_filename_from_utf8(str->str,strlen(str->str),&rb,&wb,NULL);
+
+      if(!conv_str)
+    	conv_str=g_strdup(str->str);
+
       if(ginfo->rel_m3u) {
-	g_snprintf(tmp,PATH_MAX,"%s",str->str);
+	g_snprintf(tmp,PATH_MAX,"%s",conv_str);
 	relnam=MakeRelative(tmp,m3unam);
 	fprintf(fp,"%s\n",relnam);
       }
       else 
-	fprintf(fp,"%s\n",str->str);
+	fprintf(fp,"%s\n",conv_str);
+      g_free(conv_str);
     }
   }
 
@@ -627,9 +649,8 @@ void KillRip(GtkWidget *widget,gpointer data)
   ginfo->all_ripdone=0;
   ginfo->all_riplast=0;
 
-  ginfo->ripping_a_disc=FALSE;
-
   ginfo->stop_rip=TRUE;
+  ginfo->ripping_a_disc=FALSE;
 
   if(ginfo->ripping) {
     /* Need to decrement num_wavs since we didn't finish ripping
@@ -711,14 +732,14 @@ static void ID3Add(GripInfo *ginfo,char *file,EncodeTrack *enc_track)
   g_snprintf(year,5,"%d",enc_track->song_year);
 
   /* If we've got id3lib, we have the option of doing v2 tags */
-#ifdef HAVE_ID3LIB
+#ifdef HAVE_ID3V2
   if(ginfo->doid3v2) {
     ID3v2TagFile(file,(*(enc_track->song_name))?enc_track->song_name:"Unknown",
 		 (*(enc_track->song_artist))?enc_track->song_artist:
 		 (*(enc_track->disc_artist))?enc_track->disc_artist:"Unknown",
 		 (*(enc_track->disc_name))?enc_track->disc_name:"Unknown",
 		 year,comment->str,enc_track->id3_genre,
-		 enc_track->track_num+1);
+		 enc_track->track_num+1,ginfo->id3v2_encoding);
   }
 #endif
   if(ginfo->doid3) {
@@ -727,7 +748,7 @@ static void ID3Add(GripInfo *ginfo,char *file,EncodeTrack *enc_track)
 		 (*(enc_track->disc_artist))?enc_track->disc_artist:"Unknown",
 		 (*(enc_track->disc_name))?enc_track->disc_name:"Unknown",
 		 year,comment->str,enc_track->id3_genre,
-		 enc_track->track_num+1);
+		 enc_track->track_num+1,ginfo->id3_encoding);
   }
 
   g_string_free(comment,TRUE);
@@ -767,6 +788,8 @@ void UpdateRipProgress(GripInfo *ginfo)
   gfloat elapsed=0;
   gfloat speed;
   gboolean result=FALSE;
+  char *conv_str;
+  gsize rb,wb;
 
   uinfo=&(ginfo->gui_info);
 
@@ -845,7 +868,7 @@ void UpdateRipProgress(GripInfo *ginfo)
 		   GTK_PIXMAP(uinfo->smile_indicator));
       }
 
-      Debug(_("Rip finished\n"));
+      LogStatus(ginfo,_("Rip finished\n"));
       ginfo->all_riplast=0;
       ginfo->ripping=FALSE;
       SetChecked(uinfo,ginfo->rip_track,FALSE);
@@ -946,7 +969,7 @@ void UpdateRipProgress(GripInfo *ginfo)
         waitpid(ginfo->mp3pid[mycpu],NULL,0);
         ginfo->encoding&=~(1<<mycpu);
 
-	Debug(_("Finished encoding on cpu %d\n"),mycpu);
+	LogStatus(ginfo,_("Finished encoding on cpu %d\n"),mycpu);
 	ginfo->all_enclast[mycpu]=0;
         gtk_progress_bar_update(GTK_PROGRESS_BAR(uinfo->mp3progbar[mycpu]),
 				0.0);
@@ -957,7 +980,17 @@ void UpdateRipProgress(GripInfo *ginfo)
 
         if(!ginfo->stop_encode) {
 	  if(ginfo->delete_wavs) {
-	    Debug(_("Deleting [%s]\n"),ginfo->rip_delete_file[mycpu]);
+            conv_str=g_filename_to_utf8(ginfo->rip_delete_file[mycpu],
+                                        strlen(ginfo->rip_delete_file[mycpu]),
+                                        &rb,&wb,NULL);
+
+            if(!conv_str)
+              conv_str=g_strdup(ginfo->rip_delete_file[mycpu]);
+
+	    LogStatus(ginfo,_("Deleting [%s]\n"),conv_str,&rb,&wb,NULL);
+
+            g_free(conv_str);
+
 	    unlink(ginfo->rip_delete_file[mycpu]);
 	  }
 
@@ -1010,7 +1043,7 @@ void UpdateRipProgress(GripInfo *ginfo)
 static void RipIsFinished(GripInfo *ginfo,gboolean aborted)
 {
   GripGUI *uinfo;
-  Debug(_("Ripping is finished\n"));
+  LogStatus(ginfo,_("Ripping is finished\n"));
 
   uinfo=&(ginfo->gui_info);
   ginfo->all_ripsize=0;
@@ -1026,6 +1059,9 @@ static void RipIsFinished(GripInfo *ginfo,gboolean aborted)
   ginfo->ripping_a_disc=FALSE;
   ginfo->rip_finished=time(NULL);
 
+  /* Re-open the cdrom device if it was closed */
+  if(ginfo->disc.cd_desc<0) CDInitDevice(ginfo->disc.devname,&(ginfo->disc));
+
   /* Do post-rip stuff only if we weren't explicitly aborted */
   if(!aborted) {
     if(ginfo->beep_after_rip) printf("%c%c",7,7);
@@ -1038,6 +1074,16 @@ static void RipIsFinished(GripInfo *ginfo,gboolean aborted)
   
     if(*ginfo->disc_filter_cmd)
       DoDiscFilter(ginfo);
+
+    if(ginfo->delayed_encoding) {
+      ginfo->encode_list = g_list_concat(ginfo->encode_list,
+                                         ginfo->pending_list);
+      ginfo->pending_list = NULL;
+      
+      /* Start an encoder on all free CPUs
+       * This is really only useful the first time through */
+      while (MP3Encode(ginfo));
+    }
     
     if(ginfo->eject_after_rip) {
       /* Reset rip_finished since we're ejecting */
@@ -1054,6 +1100,8 @@ char *TranslateSwitch(char switch_char,void *data,gboolean *munge)
 {
   static char res[PATH_MAX];
   EncodeTrack *enc_track;
+  gchar *conv_str,*st;
+  gsize rb,wb;
 
   enc_track=(EncodeTrack *)data;
 
@@ -1144,6 +1192,10 @@ char *TranslateSwitch(char switch_char,void *data,gboolean *munge)
     *munge=FALSE;
     break;
 #endif
+  case 'x':
+    g_snprintf(res,PATH_MAX,"%s",enc_track->ginfo->mp3extension);
+    *munge=FALSE;
+    break;
   default:
     *res='\0';
     break;
@@ -1220,6 +1272,9 @@ void DoRip(GtkWidget *widget,gpointer data)
 
   CDStop(&(ginfo->disc));
   ginfo->stopped=TRUE;
+
+  /* Close the device so as not to conflict with ripping */
+  CDCloseDevice(&(ginfo->disc));
     
   if(ginfo->ripping) {
     ginfo->doencode=FALSE;
@@ -1250,8 +1305,12 @@ void DoRip(GtkWidget *widget,gpointer data)
     return;
   }
   
+  ginfo->stop_rip=FALSE;
+
   CalculateAll(ginfo);
+
   result=RipNextTrack(ginfo);
+
   if(!result) {
     ginfo->doencode=FALSE;
   }
@@ -1297,6 +1356,9 @@ static gboolean RipNextTrack(GripInfo *ginfo)
   struct stat mystat;
   GString *str;
   EncodeTrack enc_track;
+  char *conv_str, *utf8_ripfile;
+  gsize rb,wb;
+  const char *charset;
 
   uinfo=&(ginfo->gui_info);
 
@@ -1319,9 +1381,9 @@ static gboolean RipNextTrack(GripInfo *ginfo)
   if(ginfo->have_disc&&ginfo->rip_track>=0) {
     Debug(_("Ripping away!\n"));
 
-    if(!ginfo->rip_partial){
+    /*    if(!ginfo->rip_partial){
       gtk_clist_select_row(GTK_CLIST(uinfo->trackclist),ginfo->rip_track,0);
-    }
+      }*/
 
     CopyPixmap(GTK_PIXMAP(uinfo->rip_pix[0]),GTK_PIXMAP(uinfo->rip_indicator));
 
@@ -1348,7 +1410,17 @@ static gboolean RipNextTrack(GripInfo *ginfo)
     TranslateString(ginfo->ripfileformat,str,TranslateSwitch,
 		    &enc_track,TRUE,&(ginfo->sprefs));
 
-    g_snprintf(ginfo->ripfile,256,"%s",str->str);
+    g_get_charset(&charset);
+
+    conv_str=g_filename_from_utf8(str->str,strlen(str->str),&rb,&wb,NULL);
+
+    if(!conv_str) {
+      conv_str=g_strdup(str->str);
+    }
+
+    g_snprintf(ginfo->ripfile,256,"%s",conv_str);
+
+    g_free(conv_str);
     g_string_free(str,TRUE);
 
     MakeDirs(ginfo->ripfile);
@@ -1362,7 +1434,13 @@ static gboolean RipNextTrack(GripInfo *ginfo)
       sleep(5);
     }
 
-    Debug(_("Ripping track %d to %s\n"),ginfo->rip_track+1,ginfo->ripfile);
+    utf8_ripfile=g_filename_to_utf8(ginfo->ripfile,strlen(ginfo->ripfile),
+                                    &rb,&wb,NULL);
+
+    if(!utf8_ripfile) utf8_ripfile=strdup(ginfo->ripfile);
+
+    LogStatus(ginfo,_("Ripping track %d to %s\n"),
+	      ginfo->rip_track+1,utf8_ripfile);
 
     ginfo->rip_started = time(NULL);
     sprintf(tmp,_("Rip: Trk %d (0.0x)"),ginfo->rip_track+1);
@@ -1370,8 +1448,11 @@ static gboolean RipNextTrack(GripInfo *ginfo)
 
     if(stat(ginfo->ripfile,&mystat)>=0) {
       if(mystat.st_size == ginfo->ripsize) { 
-	Debug(_("File %s has already been ripped. Skipping...\n"),\
-	      ginfo->ripfile);
+	LogStatus(ginfo,_("File %s has already been ripped. Skipping...\n"),\
+	          utf8_ripfile);
+
+        g_free(utf8_ripfile);
+
 	if(ginfo->doencode) ginfo->num_wavs++;
 	ginfo->ripping=TRUE;
 	ginfo->ripping_a_disc=TRUE;
@@ -1382,6 +1463,8 @@ static gboolean RipNextTrack(GripInfo *ginfo)
       else unlink(ginfo->ripfile);
     }
 
+    g_free(utf8_ripfile);
+
     bytesleft=BytesLeftInFS(ginfo->ripfile);
 
     if(bytesleft<(ginfo->ripsize*1.5)) {
@@ -1405,6 +1488,8 @@ static gboolean RipNextTrack(GripInfo *ginfo)
       MakeTranslatedArgs(ginfo->ripcmdline,args,100,TranslateSwitch,
 			 &enc_track,FALSE,&(ginfo->sprefs));
 
+      ArgsToLocale(args);
+
       for(arg=0;args[arg];arg++) {
 	char_args[arg+1]=args[arg]->str;
       }
@@ -1413,6 +1498,9 @@ static gboolean RipNextTrack(GripInfo *ginfo)
 
       char_args[0]=FindRoot(ginfo->ripexename);
 
+      ginfo->curr_pipe_fd=
+	GetStatusWindowPipe(ginfo->gui_info.rip_status_window);
+
       ginfo->rippid=fork();
       
       if(ginfo->rippid==0) {
@@ -1420,9 +1508,12 @@ static gboolean RipNextTrack(GripInfo *ginfo)
 	nice(ginfo->ripnice);
 	execv(ginfo->ripexename,char_args);
 	
-	Debug(_("Exec failed\n"));
+	LogStatus(ginfo,_("Exec failed\n"));
 	_exit(0);
       }
+      else {
+	ginfo->curr_pipe_fd=-1;
+      }
 
       for(arg=0;args[arg];arg++) {
 	g_string_free(args[arg],TRUE);
@@ -1446,6 +1537,8 @@ static void ThreadRip(void *arg)
 {
   GripInfo *ginfo;
   int paranoia_mode;
+  int dup_output_fd;
+  FILE *output_fp;
 
   ginfo=(GripInfo *)arg;
 
@@ -1468,12 +1561,19 @@ static void ThreadRip(void *arg)
   
   nice(ginfo->ripnice);
 
+  dup_output_fd=dup(GetStatusWindowPipe(ginfo->gui_info.rip_status_window));
+  output_fp=fdopen(dup_output_fd,"w");
+  setlinebuf(output_fp);
+
   CDPRip(ginfo->cd_device,ginfo->force_scsi,ginfo->rip_track+1,
 	 ginfo->start_sector,
 	 ginfo->end_sector,ginfo->ripfile,paranoia_mode,
 	 &(ginfo->rip_smile_level),&(ginfo->rip_percent_done),
-	 &(ginfo->stop_thread_rip_now),ginfo->calc_gain);
-  
+	 &(ginfo->stop_thread_rip_now),ginfo->calc_gain,
+	 output_fp);
+
+  fclose(output_fp);
+ 
   ginfo->in_rip_thread=FALSE;
 
   pthread_exit(0);
@@ -1523,9 +1623,13 @@ static void AddToEncode(GripInfo *ginfo,int track)
   FillInTrackInfo(ginfo,track,new_track);
   strcpy(new_track->wav_filename,ginfo->ripfile);
 
-  ginfo->encode_list=g_list_append(ginfo->encode_list,new_track);
+  if (!ginfo->delayed_encoding)
+    ginfo->encode_list=g_list_append(ginfo->encode_list,new_track);
+  else
+    ginfo->pending_list=g_list_append(ginfo->pending_list,new_track);
 
-  Debug(_("Added track %d to encode list\n"),track+1);
+  Debug(_("Added track %d to %s list\n"),track+1,
+          ginfo->delayed_encoding ? "pending" : "encoding");
 }
 
 static gboolean MP3Encode(GripInfo *ginfo)
@@ -1540,6 +1644,8 @@ static gboolean MP3Encode(GripInfo *ginfo)
   GString *str;
   int encode_track;
   int cpu;
+  char *conv_str;
+  gsize rb,wb;
 
   uinfo=&(ginfo->gui_info);
 
@@ -1573,7 +1679,14 @@ static gboolean MP3Encode(GripInfo *ginfo)
   TranslateString(ginfo->mp3fileformat,str,TranslateSwitch,
 		  enc_track,TRUE,&(ginfo->sprefs));
 
-  g_snprintf(ginfo->mp3file[cpu],256,"%s",str->str);
+  conv_str=g_filename_from_utf8(str->str,strlen(str->str),&rb,&wb,NULL);
+
+  if(!conv_str)
+    conv_str=g_strdup(str->str);
+
+  g_snprintf(ginfo->mp3file[cpu],256,"%s",conv_str);
+
+  g_free(conv_str);
   g_string_free(str,TRUE);
 
   MakeDirs(ginfo->mp3file[cpu]);
@@ -1584,7 +1697,14 @@ static gboolean MP3Encode(GripInfo *ginfo)
   
   bytesleft=BytesLeftInFS(ginfo->mp3file[cpu]);
   
-  Debug(_("%i: Encoding to %s\n"),cpu+1,ginfo->mp3file[cpu]);
+  conv_str=g_filename_to_utf8(ginfo->mp3file[cpu],strlen(ginfo->mp3file[cpu]),
+                              &rb,&wb,NULL);
+
+  if(!conv_str) conv_str=strdup(ginfo->mp3file[cpu]);
+
+  LogStatus(ginfo,_("%i: Encoding to %s\n"),cpu+1,conv_str);
+
+  g_free(conv_str);
   
   sprintf(tmp,_("Enc: Trk %d (0.0x)"),encode_track+1);
   gtk_label_set(GTK_LABEL(uinfo->mp3_prog_label[cpu]),tmp);
@@ -1605,6 +1725,8 @@ static gboolean MP3Encode(GripInfo *ginfo)
 
   MakeTranslatedArgs(ginfo->mp3cmdline,args,100,TranslateSwitch,
 		     enc_track,FALSE,&(ginfo->sprefs));
+
+  ArgsToLocale(args);
   
   for(arg=0;args[arg];arg++) {
     char_args[arg+1]=args[arg]->str;
@@ -1614,6 +1736,9 @@ static gboolean MP3Encode(GripInfo *ginfo)
   
   char_args[0]=FindRoot(ginfo->mp3exename);
 
+  ginfo->curr_pipe_fd=
+    GetStatusWindowPipe(ginfo->gui_info.encode_status_window);
+
   ginfo->mp3pid[cpu]=fork();
   
   if(ginfo->mp3pid[cpu]==0) {
@@ -1623,6 +1748,9 @@ static gboolean MP3Encode(GripInfo *ginfo)
     execv(ginfo->mp3exename,char_args);
     _exit(0);
   }
+  else {
+    ginfo->curr_pipe_fd=-1;
+  }
 
   for(arg=0;args[arg];arg++) {
     g_string_free(args[arg],TRUE);
